/*-----------------------------------------------------------------------

File  : cco_simplification.c

Author: Stephan Schulz

Contents
 
  Control of simplified clauses

Changes

<1> Mon Jun  8 14:49:49 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "cco_simplification.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


HCB_p DemodOrderCriterion = NULL;


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: DemodInsert()
//
//   Insert a demodulator into the set...
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

void DemodInsert(ClauseSet_p set, Clause_p new)
{
   assert(!new->evaluations);

   if(!set->demod_index)
   {
      assert(DemodOrderCriterion);
      HCBClauseEvaluate(DemodOrderCriterion, new);
      ClauseSetInsert(set, new);
   }
   else
   {
      ClauseSetIndexedInsert(set, new);
   }
}


/*-----------------------------------------------------------------------
//
// Function: DemodDelete()
//
//   Delete a demodulator from the set...
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

void DemodDelete(Clause_p new)
{
   assert(new->set);

   if(!(new->set->demod_index))
   {
      ClauseSetDeleteEntry(new);
   }
   else
   {
      ClauseSetIndexedDeleteEntry(new);
   }
}


/*-----------------------------------------------------------------------
//
// Function: ClauseRemoveOrientation()
//
//   Set literal types of left-rewritten literals to EqUnoriented.
//
// Global Variables: -
//
// Side Effects    : Changes orientation of equations in clause
//
/----------------------------------------------------------------------*/

void ClauseRemoveOrientation(Clause_p clause)
{
   EqnListRemoveOrientation(clause->literals);
}


/*-----------------------------------------------------------------------
//
// Function: RemoveClausesWithRewritableMaxSides()
//
//   Remove all clauses for which potentially maximal terms can be
//   rewritten with new_demod and store copies with new terms from
//   new_terms into into. Return number of clauses deleted.
//
// Global Variables: 
//
// Side Effects    : 
//
/----------------------------------------------------------------------*/

long RemoveClausesWithRewritableMaxSides(OCB_p ocb, ClauseSet_p from,
					 ClauseSet_p into, TB_p
					 new_bank, Clause_p
					 new_demod, SysDate nf_date)
{
   PTree_p  store = NULL;
   PStack_p stack = PStackAlloc();
   Clause_p handle, new;
   PTree_p  cell;
   long     res = 0;

   FindClausesWithRewritableMaxSides(ocb, from, &store, new_demod,
				     nf_date);
   
   PStackPushP(stack, store);
   while(!PStackEmpty(stack))
   {
      cell = PStackPopP(stack);
      if(cell)
      {
	 PStackPushP(stack, cell->lson);
	 PStackPushP(stack, cell->rson);
	 handle = cell->key;	 

	 res++;
	 ClauseKillChildren(handle);
	 new = ClauseCopy(handle, new_bank);
	 DemodDelete(handle);
	 ClauseSetInsert(into, new);
      }
   }
   PStackFree(stack);
   PTreeFree(store);

   return res;
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


